@@ -3,7 +3,7 @@
  *****************************************************************************
  *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  Copyright (C) 2008-2010 Lawrence Livermore National Security.
- *  Portions Copyright (C) 2010-2018 SchedMD LLC <https://www.schedmd.com>
+ *  Copyright (C) SchedMD LLC.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Mark Grondona <grondona1@llnl.gov>, et. al.
  *  CODE-OCEC-09-009. All rights reserved.
@@ -62,7 +62,7 @@
 #include "src/common/proc_args.h"
 #include "src/interfaces/mpi.h"
 #include "src/common/slurm_protocol_api.h"
-#include "src/common/slurm_protocol_interface.h"
+#include "src/common/slurm_protocol_socket.h"
 #include "src/common/slurm_rlimits_info.h"
 #include "src/common/slurm_resource_info.h"
 #include "src/interfaces/acct_gather_profile.h"
@@ -85,6 +85,7 @@ static void _usage(void);
 static void _autocomplete(const char *query);
 
 /*---- global variables, defined in opt.h ----*/
+int colon_cnt = 0;
 int	error_exit = 1;
 int	immediate_exit = 1;
 srun_opt_t sropt;
@@ -94,13 +95,15 @@ slurm_opt_t opt = {
 	.usage_func = _usage,
 	.autocomplete_func = _autocomplete,
 };
-List 	opt_list = NULL;
+list_t *opt_list = NULL;
 int	pass_number = 0;
 time_t	srun_begin_time = 0;
 bool local_het_step = false;
 
 /*---- forward declarations of static variables and functions  ----*/
 
+static bool is_step = false;
+
 static slurm_opt_t *_get_first_opt(int het_job_offset);
 static slurm_opt_t *_get_next_opt(int het_job_offset, slurm_opt_t *opt_last);
 
@@ -131,7 +134,7 @@ static bool  _valid_node_list(char **node_list_pptr);
  */
 static slurm_opt_t *_get_first_opt(int het_job_offset)
 {
-	ListIterator opt_iter;
+	list_itr_t *opt_iter;
 	slurm_opt_t *opt_local;
 
 	if (!opt_list) {
@@ -167,7 +170,7 @@ static slurm_opt_t *_get_first_opt(int het_job_offset)
  */
 static slurm_opt_t *_get_next_opt(int het_job_offset, slurm_opt_t *opt_last)
 {
-	ListIterator opt_iter;
+	list_itr_t *opt_iter;
 	slurm_opt_t *opt_local;
 	bool found_last = false;
 
@@ -224,7 +227,7 @@ extern slurm_opt_t *get_next_opt(int het_job_offset)
  */
 extern int get_max_het_group(void)
 {
-	ListIterator opt_iter;
+	list_itr_t *opt_iter;
 	slurm_opt_t *opt_local;
 	int max_het_job_offset = 0, het_job_offset = 0;
 
@@ -297,6 +300,7 @@ static slurm_opt_t *_opt_copy(void)
 	opt_dup->ifname = xstrdup(opt.ifname);
 	opt_dup->job_name = xstrdup(opt.job_name);
 	opt.licenses = NULL;		/* Moved by memcpy */
+	opt.resources = NULL; /* Moved by memcpy */
 	opt.mail_user = NULL;		/* Moved by memcpy */
 	opt_dup->mcs_label = xstrdup(opt.mcs_label);
 	opt.mem_bind = NULL;		/* Moved by memcpy */
@@ -342,6 +346,8 @@ extern int initialize_and_process_args(int argc, char **argv, int *argc_off)
 	bool opt_found = false;
 	static bool check_het_step = false;
 
+	is_step = getenv("SLURM_JOB_ID") ? true : false;
+
 	het_grp_bits = _get_het_group(argc, argv, default_het_job_offset++,
 				      &opt_found);
 	/*
@@ -371,7 +377,7 @@ extern int initialize_and_process_args(int argc, char **argv, int *argc_off)
 		_opt_default();
 
 		/* do not set adjust defaults in an active allocation */
-		if (!getenv("SLURM_JOB_ID")) {
+		if (!is_step) {
 			bool first = (pass_number == 1);
 			if (cli_filter_g_setup_defaults(&opt, first)) {
 				error("cli_filter plugin terminated with error");
@@ -381,7 +387,9 @@ extern int initialize_and_process_args(int argc, char **argv, int *argc_off)
 
 		if (opt_found || (i > 0)) {
 			xstrfmtcat(sropt.het_group, "%d", i);
-			sropt.het_grp_bits = bit_alloc(MAX_HET_JOB_COMPONENTS);
+			if (!sropt.het_grp_bits)
+				sropt.het_grp_bits =
+					bit_alloc(MAX_HET_JOB_COMPONENTS);
 			bit_set(sropt.het_grp_bits, i);
 		}
 
@@ -409,7 +417,7 @@ extern int initialize_and_process_args(int argc, char **argv, int *argc_off)
 			 * trying to use the whole allocation.
 			 */
 			if (!getenv("SLURM_HET_SIZE") &&
-			    getenv("SLURM_JOB_ID") &&
+			    is_step &&
 			    (optind >= 0) && (optind < argc)) {
 				for (int i2 = optind; i2 < argc; i2++) {
 					if (!xstrcmp(argv[i2], ":")) {
@@ -450,7 +458,7 @@ extern int initialize_and_process_args(int argc, char **argv, int *argc_off)
 		if (opt.verbose)
 			slurm_print_set_options(&opt);
 
-		if (spank_init_post_opt() < 0) {
+		if (spank_init_post_opt()) {
 			error("Plugin stack post-option processing failed.");
 			exit(error_exit);
 		}
@@ -542,6 +550,7 @@ env_vars_t env_vars[] = {
   { "SLURM_CLUSTERS", 'M' },
   { "SLURM_CLUSTER_CONSTRAINT", LONG_OPT_CLUSTER_CONSTRAINT },
   { "SLURM_COMPRESS", LONG_OPT_COMPRESS },
+  { "SLURM_CONSOLIDATE_SEGMENTS", LONG_OPT_CONSOLIDATE_SEGMENTS },
   { "SLURM_CONSTRAINT", 'C' },
   { "SLURM_CORE_SPEC", 'S' },
   { "SLURM_CPUS_PER_TASK", 'c' },
@@ -589,6 +598,7 @@ env_vars_t env_vars[] = {
   { "SLURM_NTASKS_PER_NODE", LONG_OPT_NTASKSPERNODE },
   { "SLURM_NTASKS_PER_GPU", LONG_OPT_NTASKSPERGPU },
   { "SLURM_NTASKS_PER_TRES", LONG_OPT_NTASKSPERTRES },
+  { "SLURM_OOM_KILL_STEP", LONG_OPT_OOMKILLSTEP },
   { "SLURM_OPEN_MODE", LONG_OPT_OPEN_MODE },
   { "SLURM_OVERCOMMIT", 'O' },
   { "SLURM_OVERLAP", LONG_OPT_OVERLAP },
@@ -604,6 +614,7 @@ env_vars_t env_vars[] = {
   { "SLURM_SEND_LIBS", LONG_OPT_SEND_LIBS },
   { "SLURM_SIGNAL", LONG_OPT_SIGNAL },
   { "SLURM_SPREAD_JOB", LONG_OPT_SPREAD_JOB },
+  { "SLURM_SPREAD_SEGMENTS", LONG_OPT_SPREAD_SEGMENTS },
   { "SLURM_SRUN_MULTI", LONG_OPT_MULTI },
   { "SLURM_STDERRMODE", 'e' }, /* Left for backward compatibility */
   { "SLURM_STDINMODE", 'i' }, /* Left for backward compatibility */
@@ -629,6 +640,7 @@ env_vars_t env_vars[] = {
   { "SRUN_ERROR", 'e' },
   { "SRUN_INPUT", 'i' },
   { "SRUN_OUTPUT", 'o' },
+  { "SRUN_SEGMENT_SIZE", LONG_OPT_SEGMENT_SIZE },
   { NULL }
 };
 
@@ -786,15 +798,8 @@ static void _opt_args(int argc, char **argv, int het_job_offset)
 	if (opt.container_id && !getenv("SLURM_CONTAINER_ID"))
 		setenvf(NULL, "SLURM_CONTAINER_ID", "%s", opt.container_id);
 
-#ifdef HAVE_NATIVE_CRAY
-	/* only fatal on the allocation */
-	if (opt.network && opt.shared && (sropt.jobid == NO_VAL))
-		fatal("Requesting network performance counters requires "
-		      "exclusive access.  Please add the --exclusive option "
-		      "to your request.");
 	if (opt.network)
-		setenv("SLURM_NETWORK", opt.network, 1);
-#endif
+		setenvf(NULL, "SLURM_NETWORK", "%s", opt.network);
 
 	if (opt.dependency)
 		setenvfs("SLURM_JOB_DEPENDENCY=%s", opt.dependency);
@@ -821,10 +826,6 @@ static void _opt_args(int argc, char **argv, int het_job_offset)
 	if (!rest && !sropt.test_only)
 		fatal("No command given to execute.");
 
-	if (launch_init() != SLURM_SUCCESS) {
-		fatal("Unable to load launch plugin, check LaunchType "
-		      "configuration");
-	}
 	command_pos = launch_g_setup_srun_opt(rest, &opt);
 
 	/* make sure we have allocated things correctly */
@@ -924,13 +925,21 @@ static bool _opt_verify(void)
 			opt.nodes_set = false;
 	}
 
+	/*
+	 * Specifying --gpus should override SLURM_GPUS_PER_NODE env if present
+	 * in step request.
+	 */
+	if (slurm_option_set_by_env(&opt, LONG_OPT_GPUS_PER_NODE) &&
+	    slurm_option_set_by_cli(&opt, 'G') && is_step)
+		slurm_option_reset(&opt, "gpus-per-node");
+
 	validate_options_salloc_sbatch_srun(&opt);
 
 	/*
 	 * If they are requesting block without 'nopack' and the system
 	 * is setup to pack nodes set it here.
 	 */
-	if ((slurm_conf.select_type_param & CR_PACK_NODES) &&
+	if ((slurm_conf.select_type_param & SELECT_PACK_NODES) &&
 	    !(opt.distribution & SLURM_DIST_NO_PACK_NODES) &&
 	    ((opt.distribution & SLURM_DIST_BLOCK) ||
 	     (opt.distribution == SLURM_DIST_UNKNOWN)))
@@ -938,14 +947,14 @@ static bool _opt_verify(void)
 
 	/*
 	 * If we are packing the nodes in an allocation set min_nodes to
-	 * 1. The slurmctld will adjust the max_nodes to the approriate
+	 * 1. The slurmctld will adjust the max_nodes to the appropriate
 	 * number if the allocation is homogeneous.
 	 */
 	if ((opt.distribution & SLURM_DIST_PACK_NODES) &&
 	    slurm_option_set_by_env(&opt, 'N')) {
 		opt.min_nodes = 1;
 		if (opt.verbose)
-			info("Reseting -N set by environment variable because of -mpack");
+			info("Resetting -N set by environment variable because of -mpack");
 		mpack_reset_nodes = true;
 	}
 
@@ -1058,8 +1067,12 @@ static bool _opt_verify(void)
 
 	/* set proc and node counts based on the arbitrary list of nodes */
 	if (((opt.distribution & SLURM_DIST_STATE_BASE) == SLURM_DIST_ARBITRARY)
-	   && (!opt.nodes_set || !opt.ntasks_set)) {
+	   && (!opt.nodes_set || !opt.ntasks_set)
+	   && !xstrchr(opt.nodelist, '{')) {
 		hostlist_t *hl = hostlist_create(opt.nodelist);
+
+		if (!hl)
+			fatal("Invalid node list specified");
 		if (!opt.ntasks_set) {
 			opt.ntasks_set = true;
 			opt.ntasks = hostlist_count(hl);
@@ -1147,12 +1160,10 @@ static bool _opt_verify(void)
 	}
 
 	/* massage the numbers */
-	if (opt.nodelist && !opt.nodes_set) {
+	if (opt.nodelist && !opt.nodes_set && !xstrchr(opt.nodelist, '{')) {
 		hl = hostlist_create(opt.nodelist);
-		if (!hl) {
-			error("memory allocation failure");
-			exit(error_exit);
-		}
+		if (!hl)
+			fatal("Invalid node list specified");
 		hostlist_uniq(hl);
 		hl_cnt = hostlist_count(hl);
 		opt.min_nodes = hl_cnt;
@@ -1183,10 +1194,8 @@ static bool _opt_verify(void)
 		if (opt.nodelist) {
 			FREE_NULL_HOSTLIST(hl);
 			hl = hostlist_create(opt.nodelist);
-			if (!hl) {
-				error("memory allocation failure");
-				exit(error_exit);
-			}
+			if (!hl)
+				fatal("Invalid node list specified");
 			if (((opt.distribution & SLURM_DIST_STATE_BASE) ==
 			     SLURM_DIST_ARBITRARY) && !opt.ntasks_set) {
 				opt.ntasks = hostlist_count(hl);
@@ -1241,19 +1250,38 @@ static bool _opt_verify(void)
 		    slurm_option_set_by_env(&opt, 'n') &&
 		    !slurm_option_set_by_env(&opt, 'N')) {
 			slurm_option_reset(&opt, "ntasks");
-		} else if ((opt.ntasks_per_node != NO_VAL) && opt.min_nodes &&
-			   (opt.ntasks_per_node !=
-			    (opt.ntasks / opt.min_nodes))) {
-			if ((opt.ntasks > opt.ntasks_per_node) &&
-			    !mpack_reset_nodes)
-				warning("can't honor --ntasks-per-node set to %u which doesn't match the requested tasks %u with the number of requested nodes %u. Ignoring --ntasks-per-node.",
+		} else if (opt.ntasks_per_node != NO_VAL) {
+			bool ntasks_per_node_reset = false;
+			int min_ntasks, max_ntasks;
+
+			min_ntasks = opt.min_nodes * opt.ntasks_per_node;
+			max_ntasks = opt.max_nodes * opt.ntasks_per_node;
+
+			/*
+			* We only want to notify incoherent combinations of
+			* -n/-N/--ntasks-per-node for steps, since job
+			* allocations will be already rejected.
+			*/
+			if (opt.max_nodes &&
+			    (opt.ntasks > max_ntasks) &&
+			    !mpack_reset_nodes &&
+			    is_step) {
+				warning("can't honor --ntasks-per-node set to %u which doesn't match the requested tasks %u with the maximum number of requested nodes %u. Ignoring --ntasks-per-node.",
 					opt.ntasks_per_node, opt.ntasks,
-					opt.min_nodes);
-			else if (opt.ntasks > opt.ntasks_per_node)
+					opt.max_nodes);
+				ntasks_per_node_reset = true;
+			}
+			else if (opt.min_nodes &&
+				 (opt.ntasks != min_ntasks) &&
+				 (opt.ntasks > opt.ntasks_per_node) &&
+				 mpack_reset_nodes) {
 				warning("can't honor --ntasks-per-node set to %u which doesn't match the requested tasks %u and -mpack, which forces min number of nodes to 1",
 					opt.ntasks_per_node, opt.ntasks);
+				ntasks_per_node_reset = true;
+			}
 
-			slurm_option_reset(&opt, "ntasks-per-node");
+			if (ntasks_per_node_reset)
+				slurm_option_reset(&opt, "ntasks-per-node");
 		}
 
 	} /* else if (opt.ntasks_set && !opt.nodes_set) */
@@ -1350,7 +1378,7 @@ extern char *spank_get_job_env(const char *name)
 
 	if ((name == NULL) || (name[0] == '\0') ||
 	    (strchr(name, (int)'=') != NULL)) {
-		slurm_seterrno(EINVAL);
+		errno = EINVAL;
 		return NULL;
 	}
 
@@ -1376,7 +1404,7 @@ extern int   spank_set_job_env(const char *name, const char *value,
 
 	if ((name == NULL) || (name[0] == '\0') ||
 	    (strchr(name, (int)'=') != NULL)) {
-		slurm_seterrno(EINVAL);
+		errno = EINVAL;
 		return -1;
 	}
 
@@ -1410,7 +1438,7 @@ extern int   spank_unset_job_env(const char *name)
 
 	if ((name == NULL) || (name[0] == '\0') ||
 	    (strchr(name, (int)'=') != NULL)) {
-		slurm_seterrno(EINVAL);
+		errno = EINVAL;
 		return -1;
 	}
 
@@ -1473,6 +1501,7 @@ static void _usage(void)
 "            [--task-prolog=fname] [--task-epilog=fname]\n"
 "            [--ctrl-comm-ifhn=addr] [--multi-prog] [--mcs-label=mcs]\n"
 "            [--cpu-freq=min[-max[:gov]]] [--power=flags] [--spread-job]\n"
+"            [--spread-segments]\n"
 "            [--switches=max-switches{@max-time-to-wait}] [--reboot]\n"
 "            [--core-spec=cores] [--thread-spec=threads]\n"
 "            [--bb=burst_buffer_spec] [--bbf=burst_buffer_file]\n"
@@ -1483,7 +1512,8 @@ static void _usage(void)
 "            [--mpi-combine=yes|no] [--het-group=value]\n"
 "            [--cpus-per-gpu=n] [--gpus=n] [--gpu-bind=...] [--gpu-freq=...]\n"
 "            [--gpus-per-node=n] [--gpus-per-socket=n] [--gpus-per-task=n]\n"
-"            [--mem-per-gpu=MB] [--tres-per-task=list]\n"
+"            [--mem-per-gpu=MB] [--tres-bind=...] [--tres-per-task=list]\n"
+"            [--oom-kill-step[=0|1]]\n"
 "            executable [args...]\n");
 
 }
@@ -1523,7 +1553,7 @@ static void _help(void)
 "      --epilog=program        run \"program\" after launching job step\n"
 "  -E, --preserve-env          env vars for node and task counts override\n"
 "                              command-line flags\n"
-"      --gres=list             required generic resources\n"
+"      --gres=list             required generic resources per node\n"
 "      --gres-flags=opts       flags related to GRES management\n"
 "  -H, --hold                  submit job in held state\n"
 "  -i, --input=in              location of stdin redirection\n"
@@ -1552,6 +1582,7 @@ static void _help(void)
 "      --nice[=value]          decrease scheduling priority by value\n"
 "      --ntasks-per-node=n     number of tasks to invoke on each node\n"
 "  -N, --nodes=N               number of nodes on which to run (N = min[-max])\n"
+"      --oom-kill-step[=0|1]   set the OOMKillStep behaviour\n"
 "  -o, --output=out            location of stdout redirection\n"
 "  -O, --overcommit            overcommit resources\n"
 "      --overlap               Allow other steps to overlap this step\n"
@@ -1579,6 +1610,7 @@ static void _help(void)
 "      --signal=[R:]num[@time] send signal when time limit within time seconds\n"
 "      --slurmd-debug=level    slurmd debug level\n"
 "      --spread-job            spread job across as many nodes as possible\n"
+"      --spread-segments       spread job segments across separate base blocks\n"
 "      --switches=max-switches{@max-time-to-wait}\n"
 "                              Optimum switches and max time to wait for optimum\n"
 "      --task-epilog=program   run \"program\" after launching task\n"
@@ -1587,11 +1619,14 @@ static void _help(void)
 "  -T, --threads=threads       set srun launch fanout\n"
 "  -t, --time=minutes          time limit\n"
 "      --time-min=minutes      minimum time limit (if distinct)\n"
+"      --tres-bind=...         task to tres binding options\n"
 "      --tres-per-task=list    list of tres required per task\n"
 "  -u, --unbuffered            do not line-buffer stdout/err\n"
 "      --use-min-nodes         if a range of node counts is given, prefer the\n"
 "                              smaller count\n"
 "  -v, --verbose               verbose mode (multiple -v's increase verbosity)\n"
+"      --wait-for-children     wait for all children processes in a task to\n"
+"                              close before considering the task ended.\n"
 "  -W, --wait=sec              seconds to wait after first task exits\n"
 "                              before killing job\n"
 "      --wckey=wckey           wckey to run job under\n"
@@ -1633,7 +1668,7 @@ static void _help(void)
 "      --sockets-per-node=S    number of sockets per node to allocate\n"
 "      --cores-per-socket=C    number of cores per socket to allocate\n"
 "      --threads-per-core=T    number of threads per core to allocate\n"
-"  -B  --extra-node-info=S[:C[:T]]  combine request of sockets per node,\n"
+"  -B, --extra-node-info=S[:C[:T]]  combine request of sockets per node,\n"
 "                              cores per socket and threads per core.\n"
 "                              Specify an asterisk (*) as a placeholder,\n"
 "                              a minimum value, or a min-max range.\n"
@@ -1670,12 +1705,6 @@ static void _help(void)
 		);
 
 	printf("\n"
-#ifdef HAVE_NATIVE_CRAY			/* Native Cray specific options */
-"Cray related options:\n"
-"      --network=type          Use network performance counters\n"
-"                              (system, network, or processor)\n"
-"\n"
-#endif
 "Help options:\n"
 "  -h, --help                  show this help message\n"
 "      --usage                 display brief usage message\n"
